--[[
**********************************************************************
MagicDKP Client - a client for the Magic DKP System.
**********************************************************************
This file is part of MagicDKP Client, a World of Warcraft Addon

MagicDKP Client is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

MagicDKP Client is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with MagicDKP Client.  If not, see <http://www.gnu.org/licenses/>.
**********************************************************************
]]

MDKP_Client = LibStub("AceAddon-3.0"):NewAddon("MagicDKP_Client", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local mod = MDKP_Client
MagicDKP_ClientDB = {}

local GUI = LibStub("AceGUI-3.0")
local MagicComm = LibStub("MagicComm-1.0")
local media = LibStub("LibSharedMedia-3.0")
local currentEvent, currentSender, currentRaid
local fmt = string.format
local networkData = {}
local pendingStandby = {}
local playerName
local tonumber = tonumber
local db = MagicDKP_ClientDB
local time = time

mod.MAJOR_VERSION = "MagicDKP_Client-1.0"
mod.MINOR_VERSION = tonumber('47') or tonumber(("$Revision: 47 $"):match("%d+"))

function mod:OnInitialize()
   playerName = UnitName("player")
   if not db.sound then
      db.sound = "Cartoon FX"
   end
   if not db.bidSound then
      db.bidSound = "Loot Chime"
   end
   StaticPopupDialogs["MDKPStandbyCheck"] = {
      text = "",
      button1 = "I am here!",
      whileDead = 1,
      hideOnEscape = 0,
      timeout = 300,
      startDelay = 0,
      delayText = "Hello",
      OnAccept = function() mod:SendStandbyAck() mod:ScheduleTimer("NextCheck", 2) end,
      OnShow = function()
		  mod.popupTimer = mod:ScheduleRepeatingTimer("UpdatePopupTimer", 1, { time(), getglobal(this:GetName().."Text") })
	       end
   }
end

function mod:OnEnable()
   MagicComm:RegisterListener(mod, "MD")
   mod:RegisterOptions()
end

function mod:OnDisable()
   MagicComm:UnregisterListener(mod, "MD")
end

local function SetNetworkData(cmd, data, misc1, misc2, misc3, misc4)
   networkData.cmd = cmd
   networkData.data = data
   networkData.misc1 = misc1
   networkData.misc2 = misc2
   networkData.misc3 = misc3
   networkData.misc4 = misc4
end

function mod:SendUrgentMessage(channel, recipient)
   MagicComm:SendUrgentMessage(networkData, "MD", channel, recipient)
end

local standbyText = "%s has initiated a standby check for %s. Standby check expiring in %ds."

function mod:UpdatePopupTimer(data)
   local timeleft = 300 - (time()-data[1])
   if timeleft > 0 then
      data[2]:SetText(standbyText:format(currentSender, currentEvent, timeleft))
   else
      mod:CancelTimer(mod.standbyTimer, true)
      StaticPopup_Hide("MDKPStandbyCheck")
      currentSender = nil
      currentEvent = nil
      currentRaid = nil
      mod:ScheduleTimer("NextCheck", 2)
   end
end


local bidFormat = "Taking bids for %s"
local bidFormatTimer = "Taking bids for %s\nBidding closes in %ss."

local function BidDialog_Cleanup()
   mod.bidDialog:Hide()
   mod.selectedPriority = nil
   mod.item = nil
   mod:CancelTimer(mod.bidTimerScheduler, true)
   mod.bidTimeExpire = nil
   mod.bidTimerScheduler = nil
   mod.bidmaster = nil
end

local function BidDialog_OnAccept(self)
   local dkp = tonumber(mod.bidDialog:GetValue())
   if dkp <= 0 then
      mod:Print("Please enter a bid value larger than zero.")
   elseif not mod.selectedPriority or mod.selectedPriority > #mod.priorities or mod.selectedPriority < 1 then
      mod:Print("You need to select the bid priority!")
   else
      SetNetworkData("DKPRESPONSE", playerName, dkp, mod.selectedPriority, mod.item)
      mod:SendUrgentMessage("WHISPER", mod.bidmaster)
      BidDialog_Cleanup()
   end
end			 

local function BidDialog_Pass()
   SetNetworkData("DKPRESPONSE", playerName, nil, nil, mod.item)
   mod:SendUrgentMessage("WHISPER", mod.bidmaster)
   BidDialog_Cleanup()
end

function mod:UpdateBidTimer()
   if not mod.bidTimeExpire then return end
   local timeleft = mod.bidTimeExpire - time()
   if timeleft > 0 then
      if not mod.bidTimerScheduler then
	 mod.bidTimerScheduler = mod:ScheduleRepeatingTimer("UpdateBidTimer", 1)
      end
      mod.bidDialog:SetText(bidFormatTimer:format(mod.item, timeleft))
   else
      BidDialog_Pass()
   end
end

-- Time to bid on an item
function mod:OnDKPBid(item, priorities, bidmaster, expirationTime)
   local dialog = mod.bidDialog
   mod.item = item
   mod.priorities = priorities
   mod.bidmaster = bidmaster

   PlaySoundFile(media:Fetch("sound", db.bidSound))

   if expirationTime then
      mod.bidTimeExpire = time() + expirationTime
   end

   if not dialog then
      dialog =  GUI:Create("MDKP_BidDialog")
      dialog:SetTitle("New Item Bid")
      dialog:SetCallback("OnBidPass", BidDialog_Pass)
      dialog:SetCallback("OnAccept", BidDialog_OnAccept)
      dialog:SetCallback("OnPrioritySelected", function(self, event, value) mod.selectedPriority = value end)
      mod.bidDialog = dialog
   end

   dialog:SetValue(0)
   dialog:SetPriorityList(priorities)
   dialog:SetItemLink(item)
   dialog:Show()
   dialog.select.frame:Show()
   mod.bidDialog:SetText(bidFormat:format(mod.item))
   mod:UpdateBidTimer()
   mod.selectedPriority = 1
end

function mod:OnDKPCancel(item, bidmaster)
   if mod.item == item and mod.bidmaster == bidmaster then
      --      mod:Print("The current bid for", item,"has been ended by", bidmaster)
      BidDialog_Pass()
   end
end

function mod:OnStandbyCheck(event, sender, raidid)
   if currentEvent then
      pendingStandby[#pendingStandby+1] = {
	 sender = sender,
	 event = event,
	 raid = raidid,
      }
   else
      mod:CancelAllTimers()
      mod:ShowStandbyCheckDialog(sender, event, raidid)
   end
end


function mod:ShowStandbyCheckDialog(sender, event, raidid)
   currentSender = sender
   currentEvent = event
   currentRaid = raidid
   StaticPopupDialogs["MDKPStandbyCheck"].text = standbyText:format(currentSender, currentEvent, 300)
   StaticPopup_Show("MDKPStandbyCheck")
   PlaySoundFile(media:Fetch("sound", db.sound))
end

function mod:SendStandbyAck()
   mod:CancelAllTimers()
   StaticPopup_Hide("MDKPStandbyCheck")
   if currentSender and currentEvent then
      SetNetworkData("STANDBYRESPONSE", playerName, currentEvent, currentRaid)
      mod:SendUrgentMessage("WHISPER", currentSender)
      currentSender = nil
      currentEvent = nil
      currentRaid = nil
   end
end

function mod:NextCheck()
   if currentEvent then return end
   local last = #pendingStandby
   if last > 0 then
      mod:ShowStandbyCheckDialog(pendingStandby[last].sender, 
				 pendingStandby[last].event, 
				 pendingStandby[last].raid)
      pendingStandby[last] = nil
   end
end

local config = {
   type = "group",
   name = "MagicDKP Client",
   args = {
      sound = {
         type = 'select',
         dialogControl = 'LSM30_Sound',
         name = 'Standby Check Sound',
         desc = 'Sound to play when a ready check is called',
         values = AceGUIWidgetLSMlists.sound,
         get = function() return db.sound end,
         set = function(_,key) db.sound = key end,
      },
      bidSound= {
         type = 'select',
         dialogControl = 'LSM30_Sound',
         name = 'DKP Bid Sound',
         desc = 'Sound to play when a new DKP bid is received',
         values = AceGUIWidgetLSMlists.sound,
         get = function() return db.bidSound end,
         set = function(_,key) db.bidSound = key end,
      }
   }
}
function mod:RegisterOptions()
   LibStub("AceConfig-3.0"):RegisterOptionsTable("MagicDKP Client", config)
   LibStub("AceConfigDialog-3.0"):AddToBlizOptions("MagicDKP Client", "MagicDKP Client")
end

-- /dump MDKP_Client:OnStandbyCheck("Event", "Helaani")
